/*  This file is part of CoverDownloader.
 *  CoverDownloader is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  CoverDownloader is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CoverDownloader; if not, write to the Free Software             
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA  */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.Configuration;
using System.Windows.Forms.Layout;


namespace AlbumArtDownloader
{
    public delegate void DelegateUpdateStatus(string status, Task t, bool savedone, Thread from);
    public delegate void DelegateUpdateProgress(int prog);
    public delegate void DelegateUpdateTaskStatus(Task t, Script s, string stat);
    public delegate void DelegateAddThumb(AlbumArtDownloader.ArtDownloader.ThumbRes res);
    public delegate void DelegateRemoveThumbs();
    public delegate void DelegateAddBrowse(string[,] items);
    public delegate void DelegateTaskDone(Task t);
    public delegate void DelegateThreadsEnded();
    public delegate void DelegatePreviewGot(Image i, AlbumArtDownloader.ArtDownloader.ThumbRes thumb);
    public delegate void DelegateDoHTML(bool b, string d);

    public partial class MainForm : Form
    {


        ArtDownloader a;
        bool waitingtoclose;
        public DelegateUpdateStatus statupdate;
        public DelegateUpdateProgress produpdate;
        public DelegateAddThumb thumbupdate;
        public DelegateRemoveThumbs removethumbs;
        public DelegatePreviewGot previewgot;
        public DelegateUpdateTaskStatus taskupdate;
        public DelegateDoHTML dohtml;
        public DelegateAddBrowse addbrowse;
        public DelegateTaskDone taskdone;
        bool stopped;
        public DelegateThreadsEnded threadended;
        private SaveWithPreset savedlg;


        string[] args;
        ListViewItem preview;
        ListViewGroup existing;
        private List<BrowserItem> itemstoadd, itemsadded;
        public MainForm(string[] argsn)
        {
            a = new ArtDownloader(this);
            a.CompileScripts();
            aborting = false;
            stopped = false;
            waitingtoclose = false;
            statupdate = new DelegateUpdateStatus(this.UpdateStatus);
            produpdate = new DelegateUpdateProgress(this.UpdateProgess);
            thumbupdate = new DelegateAddThumb(this.callbackAddThumb);
            removethumbs = new DelegateRemoveThumbs(this.RemoveThumbs);
            previewgot = new DelegatePreviewGot(this.PreviewGot);
            threadended = new DelegateThreadsEnded(this.ThreadsEnded);
            taskupdate = new DelegateUpdateTaskStatus(this.TaskUpdate);
            taskdone = new DelegateTaskDone(this.TaskDone);
            dohtml = new DelegateDoHTML(this.DoHTML);
            itemstoadd = new List<BrowserItem>();
            itemsadded = new List<BrowserItem>();
            InitializeComponent();
            lvwColumnSorter = new ListViewColumnSorter();
            browser.ListViewItemSorter = lvwColumnSorter;
            args = argsn;

        }
        public void DoHTML(bool b, string d)
        {
            new Form2(b, d).Show();
        }
        public unsafe struct COPYDATASTRUCT
        {
            public int dwData;
            public int cbData;
            public int lpData;
        }
        protected override void WndProc(ref Message m)
        {

            if (m.Msg == 74)
            {
                unsafe
                {
                    COPYDATASTRUCT data = *((COPYDATASTRUCT*)m.LParam);
                    if (data.dwData == 7)
                    {
                        string s = new string((char*)(data.lpData), 0, data.cbData / sizeof(char));
                        ProcessArgs(s.Split('\0'), false);
                    }
                }
            }
            base.WndProc(ref m);
        }
        public void TaskUpdate(Task t, Script s, string stat)
        {

            t.listviewitem.SubItems[s.listcol.Index].Text = stat;

        }
        public void TaskDone(Task t)
        {
            lock (t.results)
            {
                if (t.results.Count == 0)
                    t.listviewitem.ImageIndex = 2;
                else
                    t.listviewitem.ImageIndex = 1;
            }
        }
        public void RemoveThumbs()
        {
            tiles.Items.Clear();
            tileimagelist.Images.Clear();
            if (Properties.Settings.Default.PreviewSavedArt)
            {
                ArtDownloader.ThumbRes r = new ArtDownloader.ThumbRes();
                Int32 width = new Int32();
                Int32 height = new Int32();
                r.Thumb = UpdateThumb(ref width, ref height);
                if (r.Thumb != null)
                {
                    r.Width = width;
                    r.Height = height;
                    r.Name = filesave.Text;
                    r.ScriptOwner = null;
                    preview = AddThumb(r);
                }
                else preview = null;
            }
            else preview = null;

        }
        bool aborting;
        public object CoolInvoke(Delegate method, params object[] args)
        {
            if (aborting || stopped) return null;
            else
                return this.Invoke(method, args);
        }
        public object CoolBeginInvoke(Delegate method, params object[] args)
        {
            if (aborting || stopped) return null;
            else
                return this.BeginInvoke(method, args);

        }
        public object CoolNoThrowBeginInvoke(Delegate method, params object[] args)
        {
            if (!aborting)
                return this.BeginInvoke(method, args);
            else return null;
        }
        public Bitmap UpdateThumb(ref Int32 width, ref Int32 height)
        {
            string s = selectedtask.FileSave;
            if (s.Length == 0 || !Properties.Settings.Default.PreviewSavedArt)
                return null;
            try
            {
                Bitmap b = new Bitmap(filesave.Text);
                if (b != null)
                {
                    Bitmap b2 = new Bitmap(b, (int)Properties.Settings.Default.ThumbnailWidth, (int)Properties.Settings.Default.ThumbnailHeight);
                    width = b.Width;
                    height = b.Height;
                    b.Dispose();
                    return b2;

                }
            }
            catch
            {
            }
            return null;
        }
        public void UpdateSize(bool refresh)
        {
            Size s = new Size((int)Properties.Settings.Default.ThumbnailWidth, (int)Properties.Settings.Default.ThumbnailHeight);
            if (tileimagelist.ImageSize != s)
            {
                tileimagelist.ImageSize = s;
                tileimagelist.Images.Clear();
                if (refresh)
                {
                    foreach (ListViewItem thumb in tiles.Items)
                    {
                        tileimagelist.Images.Add(((ArtDownloader.ThumbRes)thumb.Tag).Thumb);
                    }
                    tiles.Refresh();
                }
            }
        }
        public void callbackAddThumb(ArtDownloader.ThumbRes r)
        {
            AddThumb(r);
        }
        public ListViewItem AddThumb(AlbumArtDownloader.ArtDownloader.ThumbRes res)
        {
            int i = tileimagelist.Images.Count;
            tileimagelist.Images.Add(res.Thumb);
            ListViewItem x;
            tiles.Items.Add(x = new ListViewItem(res.ScriptOwner != null ? res.Name : "Existing", i, res.ScriptOwner != null ? res.ScriptOwner.group : existing));
            //x.ToolTipText = res.width.ToString() + "x" + res.height.ToString();
            if (res.Exact && Properties.Settings.Default.ExactMatchBold)
                x.Font = new Font(x.Font, FontStyle.Bold);
            x.Tag = res;
            tiles.Update();
            return x;
        }
        public ListViewGroup AddGroup(AlbumArtDownloader.Script s)
        {
            ListViewGroup x = new ListViewGroup(s.Name);
            tiles.Groups.Add(x);
            return x;
        }
        public void ThumbClicked(AlbumArtDownloader.ArtDownloader.ThumbRes res, bool usedefsave)
        {
            if (filesave.Text.Length == 0 || !usedefsave)
            {
                if (GetFileName())
                {
                    a.SaveArt(res, filesave.Text);
                }
            }
            else a.SaveArt(res, filesave.Text);
        }
        public void UpdateProgess(int prog)
        {
            if (prog == -1) MainProgress.Visible = false;
            else
            {
                System.Diagnostics.Debug.WriteLine(prog.ToString() + "%");
                MainProgress.Visible = true;
                MainProgress.Minimum = 0;
                MainProgress.Maximum = 100;
                MainProgress.Value = prog;
            }
        }
        public void PreviewGot(Image i, AlbumArtDownloader.ArtDownloader.ThumbRes s)
        {
            UpdateStatus("Image Retrieved", null, false, null);
            ShowPreview(i, s);
        }
        public void ThreadsEnded()
        {
            if (waitingtoclose && !a.ThreadsActive()) this.Close();
        }
        public void UpdateStatus(string status, Task t, bool savedone, Thread from)
        {
            MainStatus.Text = status;
            if (savedone)
            {
                MainProgress.Visible = false;
                if (t != null)
                {
                    PictureChosen(t);
                }
                /*       if (tiles.Count != 0)
                       {
                           if (preview != null)
                           {
                               Int32 width = new Int32();
                               Int32 height = new Int32();
                               Bitmap b = UpdateThumb(ref width, ref height);
                               //preview.SetThumb(b,width,height);
                           }
                       }*/

            }
        }

        private void PictureChosen(Task t)
        {
            if (Properties.Settings.Default.CloseAfterSaving)
            {
                a.RemoveTask(t);
                if (queue.Items.Count == 0)
                    this.Close();
            }
        }

        private void button1_Click(object sender, EventArgs e)
        {
            String art = artist.Text;
            String alb = album.Text;
            string fz = filesave.Text;
            artist.Focus();
            Task t;
            a.AddTask(t = new Task(a, art, alb, fz, (Control.ModifierKeys == Keys.Control)));
            SwitchToTask(t);
        }
        /*      private void DrawHighlight(Graphics g)
              {
                  Rectangle r=new Rectangle(highx*160+9,highy*160+9,151,151);
                  g.DrawRectangle(Pens.Red, r);

              }*/

        private void PreviewPanel_Paint(object sender, PaintEventArgs e)
        {
            /*      int i = 0;
                  int n = 0;
                  Rectangle dest = new Rectangle(0,0,150,150);
                  System.Diagnostics.Debug.Write("waiting for lock...");
                  lock (a)
                  {
                      System.Diagnostics.Debug.WriteLine("got it.");
                      //PreviewPanel.Size = new Size(a.results.Count * 150, 100);
                      System.Diagnostics.Debug.Write("painting " + a.results.Count);
                      foreach (walmartdownload.AlbumArtDownloader.ThumbRes r in a.results)
                      {
                          dest.X = 10 + 160 * i;
                          dest.Y = 10 + 160 * n;
                          //e.Graphics.DrawRectangle(Pens.Black, dest);
                          //e.Graphics.DrawString(i.ToString() + "," + n.ToString(), Font, Brushes.Red, dest.X, dest.Y);
                          e.Graphics.DrawImage(r.thumb, dest);
                          System.Diagnostics.Debug.WriteLine("Drew " + i.ToString() + "," + n.ToString());
                          i++;
                          if (i >= across)
                          {
                              i = 0;
                              n++;
                          }
                              
                      }
                      DrawHighlight(e.Graphics);
                  }*/
        }

        private void label1_Click(object sender, EventArgs e)
        {

        }

        private void PreviewPanel_MouseEnter(object sender, EventArgs e)
        {

        }

        private void PreviewPanel_Click(object sender, EventArgs e)
        {
        }

        private void toolStripStatusLabel1_Click(object sender, EventArgs e)
        {

        }
        public void ScriptsLoaded(List<Script> scripts)
        {
            string[] strings;
            lock (scripts)
            {
                strings = new string[scripts.Count];
                int i = 0;
                foreach (Script s in scripts)
                {
                    strings[i++] = "0";
                }
            }
            foreach (ListViewItem i in queue.Items)
            {
                i.SubItems.AddRange(strings);
            }
        }


        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            BrowserWorker.CancelAsync();
            if (a.ThreadsActive() || BrowserWorker.IsBusy)
            {
                e.Cancel = true;
                a.Abort();
                aborting = true;
                MainStatus.Text = "Waiting for threads...";
                waitingtoclose = true;
                timer1.Enabled = true;
                this.Enabled = false;
            }
            else
            {
                Properties.Settings.Default.MainWindowState = WindowState;
                if (WindowState != FormWindowState.Normal)
                    WindowState = FormWindowState.Normal;
                Properties.Settings.Default.MainPosX = DesktopBounds.X;
                Properties.Settings.Default.MainPosY = DesktopBounds.Y;
                Properties.Settings.Default.MainPosW = DesktopBounds.Width;
                Properties.Settings.Default.MainPosH = DesktopBounds.Height;
                Properties.Settings.Default.ShowQueue = checkBox2.Checked;
                Properties.Settings.Default.ShowBrowser = checkBox1.Checked;
                if (!Properties.Settings.Default.ShowQueue)
                {
                    checkBox2.Checked = true;

                }
                if (!Properties.Settings.Default.ShowBrowser)
                {
                    checkBox1.Checked = true;

                }

                Properties.Settings.Default.SplitPos = splitContainer4.SplitterDistance;
                Properties.Settings.Default.BrowseSplitPos = splitContainer3.SplitterDistance;
                Properties.Settings.Default.QueueCols = SaveListViewSizeToString(queue);
                Properties.Settings.Default.BrowserCols = SaveListViewSizeToString(browser);
                Properties.Settings.Default.Save();
            }
        }
        internal void SizeListViewFromString(string str, ListView list)
        {
            string[] colwidths = str.Split(';');
            int i = 0;
            foreach (string s in colwidths)
            {
                int width;
                if (Int32.TryParse(s, out width))
                {
                    if (i < list.Columns.Count)
                        list.Columns[i++].Width = width;
                }
            }
        }
        internal string SaveListViewSizeToString(ListView list)
        {
            string[] colwidths = new string[list.Columns.Count];
            int i = 0;
            foreach (ColumnHeader c in list.Columns)
            {
                colwidths[i++] = c.Width.ToString();
            }
            return string.Join(";", colwidths);
        }
        private void Form1_Load(object sender, EventArgs e)
        {
            bool forceminimise = false;
            a.TaskAdded += new TaskEventHandler(a_TaskAdded);
            a.TaskRemoved += new TaskEventHandler(a_TaskRemoved);
            a.ThreadCountChanged += new ThreadEventHandler(a_ThreadCountChanged);
            savedlg = new SaveWithPreset();
            savedlg.DefaultExt = "jpg";
            tiles.SetExStyle();
            browser.SetExStyleNoBorderSelect();
            UpdateSize(false);
            tiles.Groups.Add(existing = new ListViewGroup("Existing"));
            a.LoadScripts();
            SizeListViewFromString(Properties.Settings.Default.QueueCols, queue);
            SizeListViewFromString(Properties.Settings.Default.BrowserCols, browser);
            a.SetThreads();
            forceminimise = ProcessArgs(args, true);
            if (Properties.Settings.Default.MainPosX != 0 || Properties.Settings.Default.MainPosY != 0 || Properties.Settings.Default.MainPosW != 0 || Properties.Settings.Default.MainPosH != 0)
            {
                Rectangle r = new Rectangle();
                r.X = Properties.Settings.Default.MainPosX;
                r.Y = Properties.Settings.Default.MainPosY;
                r.Width = Properties.Settings.Default.MainPosW;
                r.Height = Properties.Settings.Default.MainPosH;
                DesktopBounds = r;
                splitContainer4.SplitterDistance = Properties.Settings.Default.SplitPos;
                splitContainer3.SplitterDistance = Properties.Settings.Default.BrowseSplitPos;
            }
            if (!forceminimise && WindowState != Properties.Settings.Default.MainWindowState)
                WindowState = Properties.Settings.Default.MainWindowState;
            checkBox2.Checked = Properties.Settings.Default.ShowQueue;
            checkBox1.Checked = Properties.Settings.Default.ShowBrowser;
            UpdateBottomPanel();
            tiles.Focus();
            if (Properties.Settings.Default.FirstRun)
            {
                (new License()).ShowDialog();
                (new SettingsDlg(this)).ShowDialog();
                Properties.Settings.Default.FirstRun = false;
                Properties.Settings.Default.Save();
            }
        }

        private bool ProcessArgs(string[] theargs, bool fromself)
        {
            bool forceminimise = false;
            if (theargs.Length > 0)
            {
                if (theargs.Length != 3)
                {
                    MessageBox.Show("Usage: albumart.exe \"<artist>\" \"<album>\" \"<path to save image>\"\r\nExample: albumart.exe \"Iron Maiden\" \"Powerslave\" \"D:\\Music\\Iron Maiden\\Powerslave\\folder.jpg\"");
                    if (fromself)
                        Close();
                }
                else
                {
                    /*if (Properties.Settings.Default.CmdLineHideTillDone && fromself)
                    {
                        WindowState = FormWindowState.Minimized;
                        forceminimise = true;
                    }*/
                    if (fromself)
                    {
                        artist.Text = theargs[0];
                        album.Text = theargs[1];
                        filesave.Text = theargs[2];
                        this.button1_Click(this, new EventArgs());
                    }
                    else
                    {
                        a.AddTask(new Task(a, theargs[0], theargs[1], theargs[2], false));
                    }
                }
            }
            return forceminimise;
        }



        void a_ThreadCountChanged(object o, ThreadEventArgs e)
        {
            if (waitingtoclose && e.count == 0)
                this.Close();
        }

        void a_TaskRemoved(object o, TaskEventArgs e)
        {
            queue.Items.Remove(e.task.listviewitem);
            if (!BlockUpdates)
            {
                if (e.task == selectedtask)
                {
                    RemoveThumbs();
                    if (queue.Items.Count > 0)
                    {
                        SwitchToTask(queue.Items[0].Tag as Task);
                    }
                }
            }

        }

        void a_TaskAdded(object o, TaskEventArgs e)
        {
            ListViewItem i = new ListViewItem(e.task.Artist);
            i.SubItems.Add(e.task.Album);
            lock (a.scripts)
            {
                foreach (Script ss in a.scripts)
                {
                    i.SubItems.Add("0");
                }
            }

            queue.Items.Add(i);
            i.Tag = e.task;
            i.ImageIndex = 0;
            lock (e.task)
            {
                e.task.listviewitem = i;
            }
            if (queue.Items.Count == 1)
                SwitchToTask(e.task);
        }

        private void groupBox2_Enter(object sender, EventArgs e)
        {

        }

        private void splitContainer1_Panel2_Layout(object sender, LayoutEventArgs e)
        {
        }

        private void PreviewPanel_MouseMove(object sender, MouseEventArgs e)
        {

        }


        private bool GetFileName()
        {
            /*        System.Windows.Forms.SaveFileDialog sfd = new System.Windows.Forms.SaveFileDialog();
                    sfd.AddExtension = true;
                    sfd.FileName = Properties.Settings.Default.SaveFileName;
                    sfd.Filter = "JPEG Files|*.jpg;*.jpeg";
                    if (sfd.ShowDialog(this) == DialogResult.OK)
                    {
                        filesave.Text = sfd.FileName;
                        return true;
                    }
                    else return false;*/
            if (savedlg.ShowDialog() == DialogResult.OK)
            {
                filesave.Text = savedlg.FileName;
                return true;
            }
            return false;


        }


        private void button2_Click(object sender, EventArgs e)
        {
            GetFileName();
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            if (waitingtoclose && !a.ThreadsActive() && !BrowserWorker.IsBusy) this.Close();
        }


        private void buttonSettings_Click(object sender, EventArgs e)
        {
            SettingsDlg d = new SettingsDlg(this);
            d.ShowDialog(this);
        }

        private void splitContainer1_Panel2_Paint(object sender, PaintEventArgs e)
        {

        }

        private void splitContainer2_Panel1_Paint(object sender, PaintEventArgs e)
        {

        }

        private void textBox1_TextChanged(object sender, EventArgs e)
        {

        }

        private void button1_Click_1(object sender, EventArgs e)
        {

        }

        private void checkBox1_CheckedChanged(object sender, EventArgs e)
        {

        }

        private void contextMenuStrip1_Opening(object sender, CancelEventArgs e)
        {

        }

        private void toolStripMenuItem1_Click(object sender, EventArgs e)
        {
            DoActivateClickThingy(false);
        }

        private void DoActivateClickThingy(bool customfname)
        {
            if (tiles.SelectedItems.Count != 0)
            {
                AlbumArtDownloader.ArtDownloader.ThumbRes t = tiles.SelectedItems[0].Tag as AlbumArtDownloader.ArtDownloader.ThumbRes;
                if (t != null && t.ScriptOwner != null)
                    ThumbClicked(t, customfname);
                PictureChosen(t.OwnerTask);
            }
        }

        private void toolStripMenuItem2_Click(object sender, EventArgs e)
        {
            OnPreviewClick();
        }

        private void OnPreviewClick()
        {
            if (tiles.SelectedItems.Count != 0)
            {
                AlbumArtDownloader.ArtDownloader.ThumbRes res = (ArtDownloader.ThumbRes)tiles.SelectedItems[0].Tag;
                if (res.ScriptOwner == null)
                {
                    ShowPreview(new Bitmap(res.Name), null);
                }
                else
                {
                    a.SaveArt(res, null);
                }
            }
        }

        private void ShowPreview(Image image, AlbumArtDownloader.ArtDownloader.ThumbRes title)
        {
            Preview p = new Preview(this, image, title);
            p.Show(this);
        }

        private void tiles_MouseClick(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Middle)
            {
                ListViewItem item = tiles.HitTest(e.Location).Item;
                if (item != null)
                {
                    tiles.SelectedItems.Clear();
                    item.Selected = true;
                    OnPreviewClick();
                }
            }
        }

        private void tiles_ItemActivate(object sender, EventArgs e)
        {
            DoActivateClickThingy(true);
        }

        private void checkBox2_CheckedChanged(object sender, EventArgs e)
        {
            UpdateBottomPanel();
        }

        private void UpdateBottomPanel()
        {
            splitContainer4.Panel2Collapsed = !checkBox2.Checked;
            splitContainer3.Panel2Collapsed = !checkBox1.Checked;
        }

        private void queue_SelectedIndexChanged(object sender, EventArgs e)
        {

        }

        private void cancelToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (queue.SelectedItems.Count == 1)
            {
                SwitchToTask(queue.SelectedItems[0].Tag as Task);
            }
        }
        Task selectedtask;
        private void SwitchToTask(Task task)
        {
            if (selectedtask != null)
            {
                selectedtask.listviewitem.Font = new Font(selectedtask.listviewitem.Font, FontStyle.Regular);
            }
            selectedtask = task;
            selectedtask.listviewitem.Font = new Font(selectedtask.listviewitem.Font, FontStyle.Bold);
            artist.Text = task.Artist;
            album.Text = task.Album;
            filesave.Text = task.FileSave;
            RemoveThumbs();
            a.SelectedTask = task;
        }

        private void cancelToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            RemoveSelectedQueueItems();
        }






        internal ColumnHeader AddCol(Script script)
        {
            return queue.Columns.Add(script.Name, 50);
        }

        private void queue_ItemActivate(object sender, EventArgs e)
        {
            SwitchToTask(queue.SelectedItems[0].Tag as Task);
        }

        internal void SetThreads()
        {
            a.SetThreads();
        }

        private void album_Enter(object sender, EventArgs e)
        {
            album.SelectAll();
        }

        private void artist_Enter(object sender, EventArgs e)
        {
            artist.SelectAll();
        }

        private void splitContainer1_SplitterMoved(object sender, SplitterEventArgs e)
        {

        }

        private void queue_ItemActivate_1(object sender, EventArgs e)
        {
            SwitchToTask(queue.SelectedItems[0].Tag as Task);
        }

        private void checkBox1_CheckedChanged_1(object sender, EventArgs e)
        {
            UpdateBottomPanel();
        }
        struct BrowserItem
        {
            public string artist;
            public string album;
            public string path;
            public bool hasart;
        }



        private void backgroundWorker1_DoWork(object sender, DoWorkEventArgs e)
        {
            try
            {
                Foobar2000Helper.ApplicationHelper07 helper;
                Foobar2000.Application07 application = null;
                helper = new Foobar2000Helper.ApplicationHelper07Class();
                if (helper != null)
                {
                    application = (Foobar2000.Application07)helper.Server;
                }
                if (application == null)
                    application = new Foobar2000.Application07Class();
                Foobar2000.Tracks07 tracks = application.MediaLibrary.GetTracks("1");
                int trackcount = tracks.Count;
                System.Collections.Generic.List<BrowserItem> albarts = new List<BrowserItem>();
                int counter = 0;
                string ext = "\\" + e.Argument as string;
                int processed = 0;
                foreach (Foobar2000.Track07 track in tracks)
                {
                    if (BrowserWorker.CancellationPending) break;
                    BrowserItem sp;
                    sp.artist = track.FormatTitle("%artist%");
                    sp.album = track.FormatTitle("%album%");
                    sp.path = System.IO.Path.GetDirectoryName(track.FormatTitle("%path%").ToLowerInvariant()) + ext;
                    sp.hasart = false;
                    ++processed;
                    if (!albarts.Contains(sp))
                    {
                        albarts.Add(sp);
                        bool b = System.IO.File.Exists(sp.path);
                        sp.hasart = b;
                        lock (itemstoadd)
                        {
                            itemstoadd.Add(sp);
                        }

                        if (counter++ == 10)
                        {
                            BrowserWorker.ReportProgress(processed * 100 / trackcount);
                            counter = 0;
                        }
                    }

                }
                if (!e.Cancel)
                    BrowserWorker.ReportProgress(100);
                helper = null;
                application = null;

            }
            catch (System.Runtime.InteropServices.COMException c)
            {
                MessageBox.Show(c.Message);
                checkBox1.Enabled = false;
                checkBox1.Checked = false;
            }
        }

        private void backgroundWorker1_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            lock (itemstoadd)
            {
                browser.BeginUpdate();
                foreach (BrowserItem b in itemstoadd)
                {
                    ListViewItem i = browser.Items.Add(b.artist);
                    i.SubItems.Add(b.album);
                    i.SubItems.Add(b.hasart ? "Yes" : "No");
                    i.ImageIndex = b.hasart ? 0 : 1;
                    i.Tag = b;
                    itemsadded.Add(b);
                }
                browser.EndUpdate();
                itemstoadd.Clear();
            }
            BrowserProgress.Value = e.ProgressPercentage;
        }




        private void backgroundWorker1_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            BrowserProgress.Visible = false;
            BrowserRefresh.Enabled = true;
            BrowserCancel.Enabled = false;
            BrowserAddArtless.Enabled = true;
            if (waitingtoclose && !a.ThreadsActive() && !BrowserWorker.IsBusy) this.Close();
        }
        private ListViewColumnSorter lvwColumnSorter;
        private void browser_ColumnClick(object sender, ColumnClickEventArgs e)
        {
            if (!BrowserWorker.IsBusy)
            {
                // Determine if clicked column is already the column that is being sorted.
                if (e.Column == lvwColumnSorter.SortColumn)
                {
                    // Reverse the current sort direction for this column.
                    if (lvwColumnSorter.Order == SortOrder.Ascending)
                    {
                        lvwColumnSorter.Order = SortOrder.Descending;
                    }
                    else
                    {
                        lvwColumnSorter.Order = SortOrder.Ascending;
                    }
                }
                else
                {
                    // Set the column number that is to be sorted; default to ascending.
                    lvwColumnSorter.SortColumn = e.Column;
                    lvwColumnSorter.Order = SortOrder.Ascending;
                }

                // Perform the sort with these new sort options.
                browser.Sort();
            }
        }

        private void browser_ItemActivate(object sender, EventArgs e)
        {
            BrowserItem b = (BrowserItem)browser.SelectedItems[0].Tag;
            a.AddTask(new Task(a, b.artist, b.album, b.path, false));
        }

        private void toolStripButton7_Click(object sender, EventArgs e)
        {
            if (BrowserRefresh.Enabled)
            {
                browser.Items.Clear();
                lock (itemstoadd)
                {
                    itemstoadd.Clear();
                    itemsadded.Clear();
                }
                BrowserWorker.RunWorkerAsync(Properties.Settings.Default.SaveFileName);
                BrowserProgress.Value = 0;
                BrowserProgress.Visible = true;
                BrowserRefresh.Enabled = false;
                BrowserCancel.Enabled = true;
                BrowserAddArtless.Enabled = false;
            }
        }

        private void BrowserCancel_Click(object sender, EventArgs e)
        {
            BrowserWorker.CancelAsync();
        }

        private void BrowserAddArtless_Click(object sender, EventArgs e)
        {
            foreach (BrowserItem b in itemsadded)
            {
                if (!b.hasart)
                {
                    a.AddTask(new Task(a, b.artist, b.album, b.path, false));
                }
            }
        }

        private void queue_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Delete)
            {
                RemoveSelectedQueueItems();
            }
        }
        private bool _BlockUpdates;
        public bool BlockUpdates
        {
            get { return _BlockUpdates; }
            set { _BlockUpdates = value; }
        }

        private void RemoveSelectedQueueItems()
        {
            bool removedselected = false;
            BlockUpdates = true;
            foreach (ListViewItem i in queue.SelectedItems)
            {
                if (i.Tag as Task == selectedtask)
                    removedselected = true;
                a.RemoveTask(i.Tag as Task);
            }
            BlockUpdates = false;
            if (removedselected)
            {
                if (queue.Items.Count > 0)
                {
                    SwitchToTask(queue.Items[0].Tag as Task);
                }
            }
        }

        private void contextMenuStrip2_Opening(object sender, CancelEventArgs e)
        {

        }

        private void saveFileDialog1_FileOk(object sender, CancelEventArgs e)
        {

        }

    }


}


